/* 

Parallelogram Class, copyright Daniele Bellutta
Disclaimer: Files imported in this class are property of their respective owners.

Description
The following class includes several methods and consturctors using ObjectDraw to 
produce a Parallelogram shape. It uses an ArrayList of lines to produce two
orthogonal grids that are partially ofset in order to produce the shape. Take
care not to use too many parallelograms in any one program because they can greatly
slow down the speed of a program.

Constructors
This class includes four constructors: two integer constructors(one that also takes 
in a color as an argument) and two double constructors (one that also takes in a 
color as an argument). Each constructor calls for three pairs of coordinates (called
x, y, a, b, w, and z in the consturctors). The arguments "x" and "y" represent the
x- and y-coordinates, respectively, of the TOP LEFT corner of the parallelogram. The
arguments "a" and "b" represent the x- and y-coordinates, respectively, of the TOP 
RIGHT corner of the parallelogram. Finally, the arguments "w" and "z" represent the
x- and y-coordinates, respectively, of the BOTTOM LEFT corner of the parallelogram.

Move Methods
The parallelogram class includes a "move()" method that moves an already-constructed 
parallelogram to a relative location. The "moveTo()" method, however, moves an 
already-constructed to an absolute location. The "moveTo()" method reimplements the 
constructor after clearing the parallelogram from the screen. Its arguments represent
the same pairs of coordinates as the constructors (explained above).

Other Methods
This class includes several other useful methods, such as a "setColor()" method, whose 
titles describe their respective purposes. There are several methods that return either 
the locations of each corner of the parallelogram or their respective x- and 
y-coordinates. The "removeFromCanvas()" removes the parallelogram both from the canvas 
and from the ArrayList of lines used to produce the parallelogram. Methods that hide and 
show the parallelogram are also included for convenience.

The "goWacko(double deg)" Method
The method "goWacko(double deg)" is a failed attempt at a method to rotate the 
parallelogram. I simply kept it in here for enjoyment.

*/

import objectdraw.*;
import java.awt.*;
import java.util.ArrayList;

public class Parallelogram
{
	ArrayList<Line> Created = new ArrayList<Line>();
	Color Stored;
	Location topleft;
	Location topright;
	Location bottomleft;
	Location bottomright;
	Location center;
	DrawingCanvas here;

	public Parallelogram(double x, double y, double a, double b, double w, double z, DrawingCanvas canvas)
	{
		if((a-x == x-w) || (y-b == z-y))
		{
			for(int i = 0; i < (z-y)+1; i++)
			{
				Created.add(new Line(x+(i*(w-x)/(z-y)),y+i,a+(i*(w-x)/(z-y)),b+i,canvas));
			}
		}
		else if((z-y) != 0)
		{				
			for(int i = 0; i < (z-y); i++)
			{
				Created.add(new Line(x+(i*(w-x)/(z-y)),y+i,a+(i*(w-x)/(z-y)),b+i,canvas));
			}
		}
		else
		{
			for(int i = 0; i < (z-y); i++)
			{
				Created.add(new Line(x,y+i,a,b+i,canvas));
			}
		}
		
		if((a-x == x-w) || (y-b == z-y))
		{
			for(int i = 0; i < (a-x); i++)
			{
				Created.add(new Line(x+1+i,y+(i*(b-y)/(a-x)),w+1+i,z+(i*(b-y)/(a-x)),canvas));
			}	
		}
		else if((a-x) != 0)
		{
			for(int i = 0; i < (a-x); i++)
			{
				Created.add(new Line(x+i,y+(i*(b-y-1)/(a-x)),w+i,z+(i*(b-y-1)/(a-x)),canvas));
			}
		}
		else
		{
			for(int i = 0; i < (a-x); i++)
			{
				Created.add(new Line(x+i,y,w+i,z,canvas));
			}
		}
		
		Stored = Color.black;
		topleft = new Location(x,y);
		topright = new Location(a,b);
		bottomleft = new Location(w,z);
		bottomright = new Location(w+(a-x),b+(z-y));
		center = new Location((a+w)/2,(b+z)/2);
		here = canvas;
	}
	
	public Parallelogram(int x, int y, int a, int b, int w, int z, DrawingCanvas canvas)
	{
		this((double)(x),(double)(y),(double)(a),(double)(b),(double)(w),(double)(z),canvas);
	}
	
	public Parallelogram(double x, double y, double a, double b, double w, double z,Color input, DrawingCanvas canvas)
	{
		this(x,y,a,b,w,z,canvas);
		setColor(input);
	}
	
	public Parallelogram(int x, int y, int a, int b, int w, int z, Color input, DrawingCanvas canvas)
	{
		this((double)(x),(double)(y),(double)(a),(double)(b),(double)(w),(double)(z),canvas);
		setColor(input);
	}
	
	public void moveTo(int x, int y, int a, int b, int w, int z)
	{
		moveTo((double)(x),(double)(y),(double)(a),(double)(b),(double)(w),(double)(z));
	}
	
	public void moveTo(double x, double y, double a, double b, double w, double z)
	{
		removeFromCanvas();
		
		if((a-x == x-w) || (y-b == z-y))
		{
			for(int i = 0; i < (z-y)+1; i++)
			{
				Created.add(new Line(x+(i*(w-x)/(z-y)),y+i,a+(i*(w-x)/(z-y)),b+i,here));
			}
		}
		else if((z-y) != 0)
		{				
			for(int i = 0; i < (z-y); i++)
			{
				Created.add(new Line(x+(i*(w-x)/(z-y)),y+i,a+(i*(w-x)/(z-y)),b+i,here));
			}
		}
		else
		{
			for(int i = 0; i < (z-y); i++)
			{
				Created.add(new Line(x,y+i,a,b+i,here));
			}
		}
		
		if((a-x == x-w) || (y-b == z-y))
		{
			for(int i = 0; i < (a-x); i++)
			{
				Created.add(new Line(x+1+i,y+(i*(b-y)/(a-x)),w+1+i,z+(i*(b-y)/(a-x)),here));
			}	
		}
		else if((a-x) != 0)
		{
			for(int i = 0; i < (a-x); i++)
			{
				Created.add(new Line(x+i,y+(i*(b-y-1)/(a-x)),w+i,z+(i*(b-y-1)/(a-x)),here));
			}
		}
		else
		{
			for(int i = 0; i < (a-x); i++)
			{
				Created.add(new Line(x+i,y,w+i,z,here));
			}
		}
		
		topleft = new Location(x,y);
		topright = new Location(a,b);
		bottomleft = new Location(w,z);
		bottomright = new Location(w+(a-x),b+(z-y));
		center = new Location((a+w)/2,(b+z)/2);
		
		setColor(Stored);
	}
	
	public void setColor(Color input)
	{
		for(int i = 0; i < Created.size(); i++)
		{
			(Created.get(i)).setColor(input);
		}
		Stored = input;
	}
	
	public Color getColor()
	{
		return Stored;
	}
	
	public void move(double x, double y)
	{
		for(int i = 0; i < Created.size(); i++)
		{
			(Created.get(i)).move(x,y);
		}
	}
	
	public void move(int x, int y)
	{
		move((double)(x),(double)(y));
	}
	
	public void removeFromCanvas()
	{
		for(int i = 0; i < Created.size(); i++)
		{
			(Created.get(i)).removeFromCanvas();
			Created.remove(i);
			i--;
		}
	}
	
	public Location getTopLeft()
	{
		return topleft;
	}
	
	public Location getTopRight()
	{
		return topright;
	}
	
	public Location getBottomLeft()
	{
		return bottomleft;
	}
	
	public Location getBottomRight()
	{
		return bottomright;
	}
	
	public Location getCenter()
	{
		return center;
	}
	
	public double getTopLeftX()
	{
		return topleft.getX();
	}
	
	public double getTopRightX()
	{
		return topright.getX();
	}
	
	public double getBottomLeftX()
	{
		return bottomleft.getX();
	}
	
	public double getBottomRightX()
	{
		return bottomright.getX();
	}

	public double getTopLeftY()
	{
		return topleft.getY();
	}
	
	public double getTopRightY()
	{
		return topright.getY();
	}
	
	public double getBottomLeftY()
	{
		return bottomleft.getY();
	}
	
	public double getBottomRightY()
	{
		return bottomright.getY();
	}
	
	public double getCenterX()
	{
		return center.getX();
	}
	
	public double getCenterY()
	{
		return center.getY();
	}
	
	public void hide()
	{
		for(int i = 0; i < Created.size(); i++)
		{
			(Created.get(i)).hide();
		}
	}
	
	public void show()
	{
		for(int i = 0; i < Created.size(); i++)
		{
			(Created.get(i)).show();
		}
	}
	
	public String toString()
	{
		String Final = "Parallelogram at ("+getTopLeftX()+","+getTopLeftY()+"), ("+getTopRightX()+","+getTopRightY()+"), ("+getBottomLeftX()+","+getBottomLeftY()+") of color "+getColor()+".";
		return Final;
	}
	
	public void goWacko(double deg)
	{
		if((getTopLeftX() >= getBottomRightX()) && (getTopLeftY() <= getBottomRightY()))
		{
			moveTo(getTopLeftX()+deg,getBottomLeftY()-deg,getTopRightX(),getTopRightY(),getBottomLeftX(),getBottomLeftY());
		}
		else if((getTopLeftX() >= getBottomRightX()) && (getTopLeftY() > getBottomRightY()))
		{
			moveTo(getTopLeftX()-deg,getBottomLeftY()-deg,getTopRightX(),getTopRightY(),getBottomLeftX(),getBottomLeftY());
		}
		else if((getTopLeftX() < getBottomRightX()) && (getTopLeftY() > getBottomRightY()))
		{
			moveTo(getTopLeftX()+deg,getBottomLeftY()-deg,getTopRightX(),getTopRightY(),getBottomLeftX(),getBottomLeftY());
		}
		else if((getTopLeftX() < getBottomRightX()) && (getTopLeftY() <= getBottomRightY()))
		{
			moveTo(getTopLeftX()+deg,getBottomLeftY()+deg,getTopRightX(),getTopRightY(),getBottomLeftX(),getBottomLeftY());
		}
		
		if((getTopRightX() >= getBottomLeftX()) && (getTopRightY() >= getBottomLeftY()))
		{
			moveTo(getTopLeftX(),getBottomLeftY(),getTopRightX()-deg,getTopRightY()+deg,getBottomLeftX()+deg,getBottomLeftY()-deg);
		}
		else if((getTopRightX() < getBottomLeftX()) && (getTopRightY() >= getBottomLeftY()))
		{
			moveTo(getTopLeftX(),getBottomLeftY(),getTopRightX()-deg,getTopRightY()-deg,getBottomLeftX()+deg,getBottomLeftY()+deg);
		}
		else if((getTopRightX() >= getBottomLeftX()) && (getTopRightY() < getBottomLeftY()))
		{
			moveTo(getTopLeftX(),getBottomLeftY(),getTopRightX()+deg,getTopRightY()+deg,getBottomLeftX()-deg,getBottomLeftY()-deg);
		}
		else if((getTopRightX() < getBottomLeftX()) && (getTopRightY() < getBottomLeftY()))
		{
			moveTo(getTopLeftX(),getBottomLeftY(),getTopRightX()+deg,getTopRightY()-deg,getBottomLeftX()-deg,getBottomLeftY()+deg);
		}
		
	}

	public void goWacko(int deg)
	{
		goWacko((double)(deg));
	}
}