/**
*Richard Cho 2011
*This class draws a Filled Triangle given the vertices of the triangle
*Version 1.1
**/
import objectdraw.*;
import java.awt.*;

public class FilledTri
{
	private Line[] lines;
	private double[] slopes;
	private Location[] locations;
	private int numlines;
	
	//accept different input
	public FilledTri(double x1, double y1, double x2, double y2, double x3, double y3, DrawingCanvas nc)
	{
		this(new Location(x1,y1),new Location(x2,y2),new Location(x3,y3),nc);
	}
	
	public FilledTri(Location point1, Location point2, Location point3, DrawingCanvas nc)
	{
		locations = new Location[3];
		slopes = new double[3];
		
		//put the points into Location array
		locations[0] = point1;
		locations[1] = point2;
		locations[2] = point3;
		
		//sort the points in increasing x coordinate order
		Location templocation = new Location(0.0,0.0);
		for(int temp = 0; temp < 2; temp++)
		{
			if(locations[temp].getX() > locations[temp+1].getX())
			{
				templocation = locations[temp];
				locations[temp] = locations[temp+1];
				locations[temp+1] = templocation;
				temp = -1;
			}
		}

		//get slopes between points
		for(int temp = 0; temp < 3; temp++)
		{
			slopes[temp] = slope(locations[temp], locations[(temp+1)%3]);
		}
		
		//start drawing them lines
		numlines = (int)locations[2].getX() - (int)locations[0].getX();
		lines = new Line[numlines];
		int count = 0;//counter for first half of triangle
		int count2 = 1;//counter for second half of triangle

		for(int temp = (int)locations[0].getX(); temp < (int)locations[2].getX(); temp++)
		{
			if(temp < (int)locations[1].getX())
			{
				lines[count] = new Line(temp,locations[0].getY() + count*slopes[0],temp,locations[0].getY() + count*slopes[2],nc);
			}
			else
			{
				lines[count] = new Line(temp,locations[1].getY() + count2*slopes[1],temp,locations[0].getY() + count*slopes[2],nc);
				count2++;
			}
			count++;
			
		}
		
		//debug help... kind of
		/*
		System.out.println("locations and slopes in order in array, ouput as X,Y,slope");
		for(int temp = 0; temp < 3; temp++)
		{
			System.out.println(locations[temp].getX() + " , " + locations[temp].getY() + " , " + slopes[temp]);
		}
		*/
		
		
	}
	
	private double slope(Location pointa, Location pointb)
	{
		if((int)(pointa.getX() - pointb.getX()) != 0)
		{
			return (pointa.getY() - pointb.getY())/(pointa.getX() - pointb.getX());
		}
		else
		{
			return 0;
		}
	}
	
	//methods
	
	//void methods
	public void move(double xmove, double ymove)
	{
		for(int temp = 0; temp < numlines; temp++)
		{
			lines[temp].move(xmove,ymove);
		}
	}
	
	//sets leftmost point at point
	public void moveTo(double xloc, double yloc)
	{
		for(int temp = 0; temp < numlines; temp++)
		{
			lines[temp].move(xloc - locations[0].getX(),yloc - locations[0].getY());
		}
	}

	public void moveTo(Location movepoint)
	{
		this.moveTo(movepoint.getX(),movepoint.getY());
	}

	public void setColor(Color dcolor)
	{
		for(int temp = 0; temp < numlines; temp++)
		{
			lines[temp].setColor(dcolor);
		}
	}

	public void hide()
	{
		for(int temp = 0; temp < numlines; temp++)
		{
			lines[temp].hide();
		}
	}

	public void show()
	{
		for(int temp = 0; temp < numlines; temp++)
		{
			lines[temp].show();
		}
	}

	public void sendForward()
	{
		for(int temp = 0; temp < numlines; temp++)
		{
			lines[temp].sendForward();
		}
	}

	public void sendBackward()
	{
		for(int temp = 0; temp < numlines; temp++)
		{
			lines[temp].sendBackward();
		}
	}

	public void sendToFront()
	{
		for(int temp = 0; temp < numlines; temp++)
		{
			lines[temp].sendToFront();
		}
	}

	public void sendToBack()
	{
		for(int temp = 0; temp < numlines; temp++)
		{
			lines[temp].sendToBack();
		}
	}

	public void removeFromCanvas()
	{
		for(int temp = 0; temp < numlines; temp++)
		{
			lines[temp].removeFromCanvas();
		}
	}
	//end void methods
	
	//return methods
	public boolean contains(Location containpoint)
	{
		int temp = 0;
		boolean contain = false;
		while(temp < numlines)
		{
			if(lines[temp].contains(containpoint))
			{
				contain = true;
				break;
			}
			temp++;
		}
		return contain;
	}

	public double getWidth()
	{
		return locations[2].getX() - locations[0].getX();
	}
	
	public double getHeight()
	{
		double[] ypoints = new double[3];
		for(int temp = 0; temp < 3; temp++)
		{
			ypoints[temp] = locations[temp].getY();
		}
		for(int temp = 0; temp < 2; temp++)
		{
			if(ypoints[temp] > ypoints[temp+1])
			{
				double dtemp = ypoints[temp];
				ypoints[temp] = ypoints[temp+1];
				ypoints[temp+1] = dtemp;
				temp = -1;
			}
		}
		return ypoints[2] - ypoints[0];
	}
	
	public Color getColor()
	{
		return lines[0].getColor();
	}
	
	public Location[] getVertices()
	{
		return locations;
	}
	//end return methods

}